<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:lav="http://software.in2p3.fr/lavoisier/config.xsd"
                xmlns="http://software.in2p3.fr/lavoisier/config.xsd"
                xmlns:x="http://www.w3.org/TR/xpath"
                exclude-result-prefixes="lav x">
    <xsl:output method="xml" indent="yes"/>
    <xsl:include href="xpath-serializer.xsl"/>
    <xsl:variable name="MATCH_FUNCTION">match</xsl:variable>
    <xsl:variable name="EVAL_FUNCTION">eval</xsl:variable>
    <xsl:variable name="RELATIVE_FUNCTIONS">|local-name|name|namespace-uri|</xsl:variable>

    <xsl:template match="/">
        <xsl:apply-templates mode="config"/>
    </xsl:template>

    <xsl:template match="lav:processor[lav:*/x:XPath]" mode="config">
        <xsl:copy><xsl:copy-of select="@*"/>
            <xsl:apply-templates mode="config" select="lav:namespace | lav:variable"/>
            <!-- evaluate once the function, except if it satisfies one of these conditions:
                 * is or contains function match()
                 * is or contains a relative function, which is not relative to a variable or a function
                 * _____ contains a relative path, which is not relative to a variable or a function
                 * is contained into a function that satisfies one of these conditions
            -->
            <xsl:for-each select="lav:*/x:XPath//x:Function
            [not(descendant-or-self::x:Function[@s1=$MATCH_FUNCTION or (@s1=$EVAL_FUNCTION and contains(x:PathExpr/x:FilterExpr/x:Literal/@s,'match()'))]
             or (descendant-or-self::x:Function[contains($RELATIVE_FUNCTIONS,concat('|',@s1,'|'))] | descendant::x:RelativeLocationPath)
             or ancestor::x:Function
                [not(descendant-or-self::x:Function[@s1=$MATCH_FUNCTION or (@s1=$EVAL_FUNCTION and contains(x:PathExpr/x:FilterExpr/x:Literal/@s,'match()'))]
                 or (descendant-or-self::x:Function[contains($RELATIVE_FUNCTIONS,concat('|',@s1,'|'))] | descendant::x:RelativeLocationPath)
                )]
            )]">
                <variable name="_{position()}">
                    <xsl:attribute name="eval">
                        <xsl:apply-templates select="."/>
                    </xsl:attribute>
                </variable>
            </xsl:for-each>
            <xsl:apply-templates mode="config" select="lav:match | lav:parameter | text() | comment()"/>
        </xsl:copy>
    </xsl:template>

    <!-- default rules -->
    <xsl:template match="@*" mode="config">
        <xsl:copy-of select="."/>
    </xsl:template>
    <xsl:template match="comment()" mode="config">
        <xsl:comment><xsl:value-of select="."/></xsl:comment>
    </xsl:template>
    <xsl:template match="*" mode="config">
        <xsl:copy>
            <xsl:apply-templates select="@*|*|text()|comment()" mode="config"/>
        </xsl:copy>
    </xsl:template>
</xsl:stylesheet>