<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xalan="http://xml.apache.org/xalan"
                xmlns:render="http://software.in2p3.fr/lavoisier/rendering.xsd">
    <xsl:output method="xml" indent="yes" xalan:indent-amount="4"/>
    <xsl:strip-space elements="*"/><!-- prevents text() function to ignore the text nodes other than 1st one -->
    <xsl:param name="view">Default rendering</xsl:param>

    <xsl:variable name="lower">abcdefghijklmnopqrstuvwxyz</xsl:variable>
    <xsl:variable name="UPPER">ABCDEFGHIJKLMNOPQRSTUVWXYZ</xsl:variable>

    <!-- rows are in level 2 -->
    <xsl:variable name="map2">map2</xsl:variable>
    <xsl:key name="map2"
             match="/*/*[@* or text()!='' or */text()!='']"
             use="concat('{',namespace-uri(),'}',local-name())"/>

    <xsl:variable name="rows2">rows2</xsl:variable>
    <xsl:key name="rows2"
             match="/*/*[key('map2',concat('{',namespace-uri(),'}',local-name()))[2]]"
             use="concat('{',namespace-uri(),'}',local-name())"/>

    <xsl:variable name="rows3">rows3</xsl:variable>
    <xsl:key name="rows3"
             match="/*/*[key('map2',concat('{',namespace-uri(),'}',local-name()))[2]]/*[text()!='']"
             use="concat('{',namespace-uri(..),'}',local-name(..), '{',namespace-uri(),'}',local-name())"/>

    <!-- rows are in level 3 -->
    <xsl:variable name="map3">map3</xsl:variable>
    <xsl:key name="map3"
             match="/*/*[not(key('map2',concat('{',namespace-uri(),'}',local-name()))[2])]/*[@* or text()!='']"
             use="concat('{',namespace-uri(..),'}',local-name(..), '{',namespace-uri(),'}',local-name())"/>

    <xsl:variable name="rows4">rows4</xsl:variable>
    <xsl:key name="rows4"
             match="/*/*[not(key('map2',concat('{',namespace-uri(),'}',local-name()))[2])]
                      /*[key('map3',concat('{',namespace-uri(..),'}',local-name(..), '{',namespace-uri(),'}',local-name()))[2]]"
             use="concat('{',namespace-uri(..),'}',local-name(..), '{',namespace-uri(),'}',local-name())"/>

    <xsl:template match="/">
        <render:rendering title="{$view}">
            <xsl:variable name="f2" select="/*/*[not(key($map2,concat('{',namespace-uri(),'}',local-name()))[2])]"/>
            <xsl:variable name="fields" select="/*/@* | /*/text()[.!=''] | $f2/@* | $f2/text()[.!=''] | $f2/*/text()[.!='']"/>
            <xsl:if test="$fields">
                <render:fields>
                    <xsl:apply-templates select="$fields" mode="field"/>
                </render:fields>
            </xsl:if>

            <xsl:variable name="r2" select="/*/*[key($map2,concat('{',namespace-uri(),'}',local-name()))[2]]"/>
            <xsl:variable name="r3" select="$f2/*[key($map3,concat('{',namespace-uri(..),'}',local-name(..), '{',namespace-uri(),'}',local-name()))[2]]"/>
            <xsl:choose>
                <!-- rows are in level 2 -->
                <xsl:when test="$r2">
                    <xsl:variable name="c2" select="/*/*[generate-id()=generate-id(key($rows2,
                        concat('{',namespace-uri(),'}',local-name()))[1])]"/>
                    <xsl:variable name="c3" select="/*/*/*[generate-id()=generate-id(key($rows3,
                        concat('{',namespace-uri(..),'}',local-name(..), '{',namespace-uri(),'}',local-name()))[1])]"/>
                    <xsl:variable name="column_names" select="$c2/@* | $c2/text()[.!=''] | $c3/text()[.!='']"/>
                    <render:rows>
                        <render:column_labels>
                            <render:column_label>Position</render:column_label>
                            <xsl:apply-templates select="$column_names" mode="column_label"/>
                        </render:column_labels>
                        <xsl:for-each select="$r2">
                            <render:row>
                                <render:column link="/lavoisier/{$view}/*/*[{position()}]">
                                    <xsl:value-of select="position()"/>
                                </render:column>
                                <xsl:apply-templates select="$column_names" mode="column">
                                    <xsl:with-param name="row" select="."/>
                                </xsl:apply-templates>
                            </render:row>
                        </xsl:for-each>
                    </render:rows>
                </xsl:when>

                <!-- rows are in level 3 -->
                <xsl:when test="$r3">
                    <xsl:variable name="c3" select="$f2/*[generate-id()=generate-id(key($rows4,
                        concat('{',namespace-uri(..),'}',local-name(..), '{',namespace-uri(),'}',local-name()))[1])]"/>
                    <xsl:variable name="column_names" select="$c3/@* | $c3/text()[.!='']"/>
                    <render:rows>
                        <render:column_labels>
                            <render:column_label>Position</render:column_label>
                            <xsl:apply-templates select="$column_names" mode="column_label"/>
                        </render:column_labels>
                        <xsl:for-each select="$r3">
                            <render:row>
                                <render:column link="/lavoisier/{$view}/*/*/*[{position()}]">
                                    <xsl:value-of select="position()"/>
                                </render:column>
                                <xsl:apply-templates select="$column_names" mode="column3">
                                    <xsl:with-param name="row3" select="."/>
                                </xsl:apply-templates>
                            </render:row>
                        </xsl:for-each>
                    </render:rows>
                </xsl:when>
            </xsl:choose>
        </render:rendering>
    </xsl:template>

    <xsl:template match="/*/text() | /*/*/text()" mode="field">
        <render:field label="{local-name(..)}">
            <xsl:value-of select="normalize-space()"/>
        </render:field>
    </xsl:template>
    <xsl:template match="/*/*/*/text()" mode="field">
        <render:field label="{concat(local-name(../..),'_',local-name(..))}">
            <xsl:value-of select="normalize-space()"/>
        </render:field>
    </xsl:template>
    <xsl:template match="/*/@*" mode="field">
        <render:field label="{local-name()}">
            <xsl:value-of select="."/>
        </render:field>
    </xsl:template>
    <xsl:template match="/*/*/@*" mode="field">
        <render:field label="{concat(local-name(..), translate(substring(local-name(),1,1),$lower,$UPPER), substring(local-name(),2))}">
            <xsl:value-of select="."/>
        </render:field>
    </xsl:template>

    <xsl:template match="/*/*/text()" mode="column_label">
        <xsl:apply-templates select="." mode="LABEL">
            <xsl:with-param name="name" select="local-name(..)"/>
        </xsl:apply-templates>
    </xsl:template>
    <xsl:template match="/*/*/*/text()" mode="column_label">
        <xsl:apply-templates select="." mode="LABEL">
            <xsl:with-param name="name" select="concat(local-name(../..),'_',local-name(..))"/>
        </xsl:apply-templates>
    </xsl:template>
    <xsl:template match="/*/*/@*" mode="column_label">
        <xsl:apply-templates select="." mode="LABEL">
            <xsl:with-param name="name" select="concat(local-name(..), translate(substring(local-name(),1,1),$lower,$UPPER), substring(local-name(),2))"/>
        </xsl:apply-templates>
    </xsl:template>
    <xsl:template match="/*/*/*/@*" mode="column_label">
        <xsl:apply-templates select="." mode="LABEL">
            <xsl:with-param name="name" select="concat(local-name(../..),'_',local-name(..), translate(substring(local-name(),1,1),$lower,$UPPER), substring(local-name(),2))"/>
        </xsl:apply-templates>
    </xsl:template>
    <xsl:template match="@* | text()" mode="LABEL">
        <xsl:param name="name"/>
        <render:column_label>
            <xsl:if test=".!='' and translate(.,'0123456789.','')=''">
                <xsl:attribute name="unit"><xsl:value-of select="$name"/></xsl:attribute>
            </xsl:if>
            <xsl:value-of select="$name"/>
        </render:column_label>
    </xsl:template>

    <xsl:template match="/*/*/text()" mode="column">
        <xsl:param name="row"/>
        <xsl:variable name="level2" select="current()/.."/>
        <xsl:variable name="value" select="normalize-space($row
            /self::*[namespace-uri()=namespace-uri($level2) and local-name()=local-name($level2)]
            /text())"/>
        <xsl:variable name="href"><xsl:apply-templates select="." mode="ANCESTORS"/>[text()='<xsl:value-of
                select="$value"/>']</xsl:variable>
        <render:column link="{$href}"><xsl:value-of select="$value"/></render:column>
    </xsl:template>
    <xsl:template match="/*/*/*/text()" mode="column">
        <xsl:param name="row"/>
        <xsl:variable name="level2" select="current()/../.."/>
        <xsl:variable name="level3" select="current()/.."/>
        <xsl:variable name="value" select="normalize-space($row
            /self::*[namespace-uri()=namespace-uri($level2) and local-name()=local-name($level2)]
            /*[namespace-uri()=namespace-uri($level3) and local-name()=local-name($level3)]
            /text())"/>
        <xsl:variable name="href">
            <xsl:apply-templates select="parent::*" mode="ANCESTORS"/>
            <xsl:text/>[<xsl:apply-templates select="parent::*" mode="SELF"/>/text()='<xsl:value-of
                select="$value"/>']</xsl:variable>
        <render:column link="{$href}"><xsl:value-of select="$value"/></render:column>
    </xsl:template>
    <xsl:template match="/*/*/@*" mode="column">
        <xsl:param name="row"/>
        <xsl:variable name="level2" select="current()/.."/>
        <xsl:variable name="attribute" select="current()"/>
        <xsl:variable name="value" select="normalize-space($row
            /self::*[namespace-uri()=namespace-uri($level2) and local-name()=local-name($level2)]
            /@*[local-name()=local-name($attribute)])"/>
        <xsl:variable name="href"><xsl:apply-templates select="." mode="ANCESTORS"/>[@<xsl:value-of
                select="local-name()"/>='<xsl:value-of select="$value"/>']</xsl:variable>
        <render:column link="{$href}"><xsl:value-of select="$value"/></render:column>
    </xsl:template>

    <xsl:template match="/*/*/*/text()" mode="column3">
        <xsl:param name="row3"/>
        <xsl:variable name="level3" select="current()/.."/>
        <xsl:variable name="value" select="normalize-space($row3
            /self::*[namespace-uri()=namespace-uri($level3) and local-name()=local-name($level3)]
            /text())"/>
        <xsl:variable name="href"><xsl:apply-templates select="." mode="ANCESTORS"/>[text()='<xsl:value-of
                select="$value"/>']</xsl:variable>
        <render:column link="{$href}"><xsl:value-of select="$value"/></render:column>
    </xsl:template>
    <xsl:template match="/*/*/*/@*" mode="column3">
        <xsl:param name="row3"/>
        <xsl:variable name="level3" select="current()/.."/>
        <xsl:variable name="attribute" select="current()"/>
        <xsl:variable name="value" select="normalize-space($row3
            /self::*[namespace-uri()=namespace-uri($level3) and local-name()=local-name($level3)]
            /@*[local-name()=local-name($attribute)])"/>
        <xsl:variable name="href"><xsl:apply-templates select="." mode="ANCESTORS"/>[@<xsl:value-of
                select="local-name()"/>='<xsl:value-of select="$value"/>']</xsl:variable>
        <render:column link="{$href}"><xsl:value-of select="$value"/></render:column>
    </xsl:template>

    <xsl:template match="* | @* | text()" mode="ANCESTORS">
        <xsl:text/>/lavoisier/<xsl:value-of select="$view"/>
        <xsl:for-each select="ancestor::*">/<xsl:apply-templates select="." mode="SELF"/></xsl:for-each>
    </xsl:template>
    <xsl:template match="*" mode="SELF">
        <xsl:choose>
            <xsl:when test="not(parent::*)">*</xsl:when>
            <xsl:when test="boolean(namespace-uri())">*[local-name()='<xsl:value-of select="local-name()"/>']</xsl:when>
            <xsl:otherwise><xsl:value-of select="local-name()"/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
</xsl:stylesheet>